@@ -1,15 +1,20 @@
 import uuid
-from typing import Optional, Union
+
+from wsproto.frame_protocol import Opcode
 
 from mitmproxy import connection
 from mitmproxy import dns
 from mitmproxy import flow
 from mitmproxy import http
 from mitmproxy import tcp
+from mitmproxy import udp
 from mitmproxy import websocket
-from mitmproxy.test.tutils import tdnsreq, tdnsresp
-from mitmproxy.test.tutils import treq, tresp
-from wsproto.frame_protocol import Opcode
+from mitmproxy.connection import ConnectionState
+from mitmproxy.proxy.mode_specs import ProxyMode
+from mitmproxy.test.tutils import tdnsreq
+from mitmproxy.test.tutils import tdnsresp
+from mitmproxy.test.tutils import treq
+from mitmproxy.test.tutils import tresp
 
 
 def ttcpflow(
@@ -35,6 +40,29 @@ def ttcpflow(
     return f
 
 
+def tudpflow(
+    client_conn=True, server_conn=True, messages=True, err=None
+) -> udp.UDPFlow:
+    if client_conn is True:
+        client_conn = tclient_conn()
+    if server_conn is True:
+        server_conn = tserver_conn()
+    if messages is True:
+        messages = [
+            udp.UDPMessage(True, b"hello", 946681204.2),
+            udp.UDPMessage(False, b"it's me", 946681204.5),
+        ]
+    if err is True:
+        err = terr()
+
+    f = udp.UDPFlow(client_conn, server_conn)
+    f.timestamp_created = client_conn.timestamp_start
+    f.messages = messages
+    f.error = err
+    f.live = True
+    return f
+
+
 def twebsocketflow(
     messages=True, err=None, close_code=None, close_reason=""
 ) -> http.HTTPFlow:
@@ -93,16 +121,17 @@ def twebsocketflow(
 
 def tdnsflow(
     *,
-    client_conn: Optional[connection.Client] = None,
-    server_conn: Optional[connection.Server] = None,
-    req: Optional[dns.Message] = None,
-    resp: Union[bool, dns.Message] = False,
-    err: Union[bool, flow.Error] = False,
+    client_conn: connection.Client | None = None,
+    server_conn: connection.Server | None = None,
+    req: dns.Message | None = None,
+    resp: bool | dns.Message = False,
+    err: bool | flow.Error = False,
     live: bool = True,
 ) -> dns.DNSFlow:
     """Create a DNS flow for testing."""
     if client_conn is None:
         client_conn = tclient_conn()
+        client_conn.proxy_mode = ProxyMode.parse("dns")
         client_conn.transport_protocol = "udp"
     if server_conn is None:
         server_conn = tserver_conn()
@@ -129,12 +158,12 @@ def tdnsflow(
 
 def tflow(
     *,
-    client_conn: Optional[connection.Client] = None,
-    server_conn: Optional[connection.Server] = None,
-    req: Optional[http.Request] = None,
-    resp: Union[bool, http.Response] = False,
-    err: Union[bool, flow.Error] = False,
-    ws: Union[bool, websocket.WebSocketData] = False,
+    client_conn: connection.Client | None = None,
+    server_conn: connection.Server | None = None,
+    req: http.Request | None = None,
+    resp: bool | http.Response = False,
+    err: bool | flow.Error = False,
+    ws: bool | websocket.WebSocketData = False,
     live: bool = True,
 ) -> http.HTTPFlow:
     """Create a flow for testing."""
@@ -185,57 +214,50 @@ def tdummyflow(client_conn=True, server_conn=True, err=None) -> DummyFlow:
 
 
 def tclient_conn() -> connection.Client:
-    c = connection.Client.from_state(
-        dict(
-            id=str(uuid.uuid4()),
-            address=("127.0.0.1", 22),
-            mitmcert=None,
-            tls_established=True,
-            timestamp_start=946681200,
-            timestamp_tls_setup=946681201,
-            timestamp_end=946681206,
-            sni="address",
-            cipher_name="cipher",
-            alpn=b"http/1.1",
-            tls_version="TLSv1.2",
-            tls_extensions=[(0x00, bytes.fromhex("000e00000b6578616d"))],
-            state=0,
-            sockname=("", 0),
-            error=None,
-            tls=False,
-            certificate_list=[],
-            alpn_offers=[],
-            cipher_list=[],
-        )
+    c = connection.Client(
+        id=str(uuid.uuid4()),
+        peername=("127.0.0.1", 22),
+        sockname=("", 0),
+        mitmcert=None,
+        timestamp_start=946681200,
+        timestamp_tls_setup=946681201,
+        timestamp_end=946681206,
+        sni="address",
+        cipher="cipher",
+        alpn=b"http/1.1",
+        tls_version="TLSv1.2",
+        state=ConnectionState.OPEN,
+        error=None,
+        tls=False,
+        certificate_list=[],
+        alpn_offers=[],
+        cipher_list=[],
+        proxy_mode=ProxyMode.parse("regular"),
     )
     return c
 
 
 def tserver_conn() -> connection.Server:
-    c = connection.Server.from_state(
-        dict(
-            id=str(uuid.uuid4()),
-            address=("address", 22),
-            source_address=("address", 22),
-            ip_address=("192.168.0.1", 22),
-            timestamp_start=946681202,
-            timestamp_tcp_setup=946681203,
-            timestamp_tls_setup=946681204,
-            timestamp_end=946681205,
-            tls_established=True,
-            sni="address",
-            alpn=None,
-            tls_version="TLSv1.2",
-            via=None,
-            state=0,
-            error=None,
-            tls=False,
-            certificate_list=[],
-            alpn_offers=[],
-            cipher_name=None,
-            cipher_list=[],
-            via2=None,
-        )
+    c = connection.Server(
+        id=str(uuid.uuid4()),
+        address=("address", 22),
+        peername=("192.168.0.1", 22),
+        sockname=("address", 22),
+        timestamp_start=946681202,
+        timestamp_tcp_setup=946681203,
+        timestamp_tls_setup=946681204,
+        timestamp_end=946681205,
+        sni="address",
+        alpn=None,
+        tls_version="TLSv1.2",
+        via=None,
+        state=ConnectionState.CLOSED,
+        error=None,
+        tls=False,
+        certificate_list=[],
+        alpn_offers=[],
+        cipher=None,
+        cipher_list=[],
     )
     return c
 
@@ -269,6 +291,9 @@ def tflows() -> list[flow.Flow]:
         tflow(ws=True),
         ttcpflow(),
         ttcpflow(err=True),
+        tudpflow(),
+        tudpflow(err=True),
         tdnsflow(resp=True),
+        tdnsflow(req=tdnsreq(questions=[])),
         tdnsflow(err=True),
     ]
